/*****************************************************************************
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: sgp.cpp,v 1.1 2000/06/19 18:05:03 kevin Exp $
**  $Log: sgp.cpp,v $
**  Revision 1.1  2000/06/19 18:05:03  kevin
**  initial cvs import
**
**  Revision 1.1  2000/06/13 16:20:31  kevin
**  finished c++ conversions
**
**  Revision 1.4  2000/05/24 22:49:01  kevin
**  Updated SGP: first function X-windows version
**
**  Revision 1.3  2000/05/11 14:07:23  kevin
**  Fixed compilation warnings
**
**  Revision 1.2  2000/05/08 20:08:15  kevin
**  *** empty log message ***
**
**  Revision 1.1.1.1  2000/04/28 13:02:44  kevin
**  Initial CVS import for first public release
**
**
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

/*-----------------------------------------------------------------------------
 * FILE IDENTIFICATION
 *
 *	Name:	    sgp.c		Simple Graphics Package
 *	Programmer: Kevin Rosenberg
 *
 *---------------------------------------------------------------------------*/

#include "stdio.h"
#include "kstddef.h"
#include "math.h"
#include "kmath.h"
#include "sgp.h"
static SGP_ID _sgp2_cwin = NULL;

extern CHARSPEC cspec;


/* NAME
 *   sgp2_init				Initialize 2 graphics system
 *
 * SYNOPSIS
 *   sgp2_init()
 */

SGP_ID
sgp2_init (int xsize, int ysize, const char *win_title )
{
    SGP_ID gid;

    gid = new SGP_WINDOW;
    _sgp2_cwin = gid;
 
    if (xsize <= 0)
	xsize = 640;
    if (ysize <= 0)
	ysize = 480;

    gid->pw_xsize = xsize;
    gid->pw_ysize = ysize;
    strncpy(gid->title, win_title, sizeof(gid->title));

    gid->recalc_mc_to_ndc = TRUE;
    gid->recalc_ndc_to_mc = TRUE;
    ident_gmtx_2 (gid->wc_to_ndc_x);
    ident_gmtx_2 (gid->mc_to_ndc_x);
    ident_gmtx_2 (gid->ndc_to_mc_x);
    ident_gmtx_2 (gid->ctm_2_x);

    sgp2_window (0., 0., 1., 1.);
    sgp2_viewport (0., 0., 1., 1.);
    ctm_clr_2 ();
    sgp2_move_abs (0., 0.);

#if HAVE_G2_H
    gid->g2_id = g2_open_X11X (gid->pw_xsize, gid->pw_ysize, 10, 10, gid->title, gid->title, NULL, -1, -1);
#endif

    _sgp2_init_dev (_sgp2_cwin);
    
    return (gid);
}


void
sgp2_close (SGP_ID gid)
{
#if HAVE_G2_H    
    g2_close (gid->g2_id);
#endif
    if (gid == _sgp2_cwin)
	_sgp2_cwin = NULL;

    delete gid;
}

void
sgp2_set_active_win (SGP_ID gid)
{
    _sgp2_cwin = gid;
}

SGP_ID
sgp2_get_active_win (void)
{
    return (_sgp2_cwin);
}


/* NAME
 *	sgp2_clear          Clear window
 */

void 
sgp2_clear ()
{
#if HAVE_G2
    if (_sgp_cwin != NULL)
	g2_clear (gid->g2_id);
#endif
}

/* NAME
 *	sgp2_window		Set window in world coordinates
 */

void
sgp2_window (double xmin, double ymin, double xmax, double ymax)
{
    if (_sgp2_cwin == NULL)
       return;

    if (xmin >= xmax || ymin >= ymax) {
	sys_error (ERR_WARNING, "Minimum > Maximum [sgp2_window]");
	return;
    }

    _sgp2_cwin->xw_min = xmin;
    _sgp2_cwin->yw_min = ymin;
    _sgp2_cwin->xw_max = xmax;
    _sgp2_cwin->yw_max = ymax;
    calc_wc_to_ndc();
}


/* NAME
 *	sgp2_viewport			Set viewport in NDC
 */

void
sgp2_viewport (double xmin, double ymin, double xmax, double ymax)
{
    if (_sgp2_cwin == NULL)
       return;

    if (xmin >= xmax || ymin >= ymax) {
	sys_error (ERR_WARNING, "Minimum > Maximum [sgp2_viewport]");
	return;
    }
    
    _sgp2_cwin->xv_min = xmin;
    _sgp2_cwin->yv_min = ymin;
    _sgp2_cwin->xv_max = xmax;
    _sgp2_cwin->yv_max = ymax;
    calc_wc_to_ndc();
    
    _sgp2_cwin->view[0] = xmin;			/* Array for clip_rect() */
    _sgp2_cwin->view[1] = ymin;
    _sgp2_cwin->view[2] = xmax;
    _sgp2_cwin->view[3] = ymax;
}


/* NAME
 *	sgp2_frame_vpt		draw box around viewport
 */

void
sgp2_frame_vpt (void)
{
    if (_sgp2_cwin == NULL)
	return;

    _sgp2_stylus (_sgp2_cwin, _sgp2_cwin->xv_min, _sgp2_cwin->yv_min, 0);
    _sgp2_stylus (_sgp2_cwin, _sgp2_cwin->xv_max, _sgp2_cwin->yv_min, 1);
    _sgp2_stylus (_sgp2_cwin, _sgp2_cwin->xv_max, _sgp2_cwin->yv_max, 1);
    _sgp2_stylus (_sgp2_cwin, _sgp2_cwin->xv_min, _sgp2_cwin->yv_max, 1);
    _sgp2_stylus (_sgp2_cwin, _sgp2_cwin->xv_min, _sgp2_cwin->yv_min, 1);
}


/* NAME
 *	calc_wc_to_ndc			Calculate transform matrix
 */

void
calc_wc_to_ndc (void)
{
    double sx, sy;
    
    if (_sgp2_cwin == NULL)
	return;

    sx = (_sgp2_cwin->xv_max - _sgp2_cwin->xv_min) / (_sgp2_cwin->xw_max - _sgp2_cwin->xw_min);
    sy = (_sgp2_cwin->yv_max - _sgp2_cwin->yv_min) / (_sgp2_cwin->yw_max - _sgp2_cwin->yw_min);
    
    ident_gmtx_2 (_sgp2_cwin->wc_to_ndc_x);
    _sgp2_cwin->wc_to_ndc_x[0][0] = sx;
    _sgp2_cwin->wc_to_ndc_x[2][0] = _sgp2_cwin->xv_min - sx * _sgp2_cwin->xw_min;
    _sgp2_cwin->wc_to_ndc_x[1][1] = sy;
    _sgp2_cwin->wc_to_ndc_x[2][1] = _sgp2_cwin->yv_min - sy * _sgp2_cwin->yw_min;
    
    _sgp2_cwin->recalc_mc_to_ndc = TRUE;
    _sgp2_cwin->recalc_ndc_to_mc = TRUE;
}


void 
calc_ndc_to_mc (void)
{
    if (_sgp2_cwin == NULL)
	return;

    if (_sgp2_cwin->recalc_mc_to_ndc) {
	mult_gmtx_2 (_sgp2_cwin->ctm_2_x, _sgp2_cwin->wc_to_ndc_x, _sgp2_cwin->mc_to_ndc_x);
	_sgp2_cwin->recalc_mc_to_ndc = FALSE;
    }
    
    invert_gmtx_2 (_sgp2_cwin->mc_to_ndc_x, _sgp2_cwin->ndc_to_mc_x);
    _sgp2_cwin->recalc_ndc_to_mc = FALSE;
}


/* NAME
 *	wc_to_ndc			Map from world coordinates to NDC
 */

void 
wc_to_ndc (double xw, double yw, double *xn, double *yn)
{
    if (_sgp2_cwin == NULL)
	return;

    if (_sgp2_cwin->recalc_mc_to_ndc) {
	mult_gmtx_2 (_sgp2_cwin->ctm_2_x, _sgp2_cwin->wc_to_ndc_x, _sgp2_cwin->mc_to_ndc_x);
	_sgp2_cwin->recalc_mc_to_ndc = FALSE;
    }

    *xn = xw * _sgp2_cwin->mc_to_ndc_x[0][0] + yw * _sgp2_cwin->mc_to_ndc_x[1][0] + _sgp2_cwin->mc_to_ndc_x[2][0];
    *yn = xw * _sgp2_cwin->mc_to_ndc_x[0][1] + yw * _sgp2_cwin->mc_to_ndc_x[1][1] + _sgp2_cwin->mc_to_ndc_x[2][1];
}


/*==============================================================*/
/* map from normalized device coords. to world coordinates	*/
/*==============================================================*/
void
ndc_to_wc (double xn, double yn, double *xw, double *yw)
{
    if (_sgp2_cwin == NULL)
	return;

    if (_sgp2_cwin->recalc_ndc_to_mc) {
	calc_ndc_to_mc();
	_sgp2_cwin->recalc_ndc_to_mc = FALSE;
    }

    *xw = xn * _sgp2_cwin->ndc_to_mc_x[0][0] + yn * _sgp2_cwin->ndc_to_mc_x[1][0] + _sgp2_cwin->ndc_to_mc_x[2][0];
    *yw = xn * _sgp2_cwin->ndc_to_mc_x[0][1] + yn * _sgp2_cwin->ndc_to_mc_x[1][1] + _sgp2_cwin->ndc_to_mc_x[2][1];
}


/*==============================================================*/
/* set the color 						*/
/*==============================================================*/
void 
sgp2_color (int icol)
{
    setcolor(icol);
}

/*==============================================================*/
/* set line style.  Pass 16 bit repeating pattern		*/
/*==============================================================*/
void 
sgp2_line_style (int style)
{
    setlinestyle(style);
}

/*==============================================================*/
/* absolute draw to						*/
/*==============================================================*/
void 
sgp2_line_abs (double x, double y)
{
    double x1, y1, x2, y2;

    if (_sgp2_cwin == NULL)
	return;

    wc_to_ndc (_sgp2_cwin->curx, _sgp2_cwin->cury, &x1, &y1);
    wc_to_ndc (x, y, &x2, &y2);

    if (clip_rect (x1, y1, x2, y2, _sgp2_cwin->view) == TRUE) { /* clip to viewport */
	_sgp2_stylus (_sgp2_cwin, x1, y1, 0);		/* move to first point */
	_sgp2_stylus (_sgp2_cwin, x2, y2, 1);		/* draw to second point */
    }

    _sgp2_cwin->curx = x;
    _sgp2_cwin->cury = y;
}

/*==============================================================*/
/* absolute move to						*/
/*==============================================================*/
void 
sgp2_move_abs (double x, double y)
{
    if (_sgp2_cwin == NULL)
	return;

    _sgp2_cwin->curx = x;
    _sgp2_cwin->cury = y;			/* moves are not clipped */
}

/*==============================================================*/
/* vector draw							*/
/*==============================================================*/
void 
sgp2_line_rel (double x, double y)
{
    if (_sgp2_cwin != NULL)
	sgp2_line_abs (x + _sgp2_cwin->curx, y + _sgp2_cwin->cury);
}

/*==============================================================*/
/* vector move							*/
/*==============================================================*/
void 
sgp2_move_rel (double x, double y)
{
    if (_sgp2_cwin != NULL)
	sgp2_move_abs (x + _sgp2_cwin->curx, y + _sgp2_cwin->cury);
}

/*==============================================================*/
/* draw text							*/
/*==============================================================*/
void 
sgp2_draw_text (char *message)
{
    double sx,sy;
    if (_sgp2_cwin == NULL)
	return;

    wc_to_ndc (_sgp2_cwin->curx, _sgp2_cwin->cury, &sx, &sy);
    _sgp2_stylus (_sgp2_cwin, sx, sy, 0);	/* move to location */
    _sgp2_dev_text (_sgp2_cwin, message);
}

void
charsize (double wid, double height)
{
    _sgp2_set_text (_sgp2_cwin, wid, height, cspec.textangle, cspec.font);
}

void
textangle (double angle)
{
    _sgp2_set_text (_sgp2_cwin, cspec.width, cspec.height, angle, cspec.font);
}

void 
sgp2_polyline_abs (double x[], double y[], int n)
{
    double x1, y1, x2, y2;
    int i;
    double xt, yt;

    if (_sgp2_cwin == NULL || n < 2)
	return;

    wc_to_ndc (x[0], y[0], &x1, &y1);
    wc_to_ndc (x[1], y[1], &x2, &y2);

    xt = x2;		/* don't pass (x2,y2) to clip, we need them */
    yt = y2;		/* as the beginning point of the next line */

    if (clip_rect (x1, y1, xt, yt, _sgp2_cwin->view) == TRUE) {
	_sgp2_stylus (_sgp2_cwin, x1, y1, 0);
	_sgp2_stylus (_sgp2_cwin, xt, yt, 1);
    }

    for (i = 2; i < n; i++) {
	x1 = x2;			/* NDC endpoint of last line */
	y1 = y2;
	wc_to_ndc (x[i], y[i], &x2, &y2);
	xt = x2;
	yt = y2;
	if (clip_rect (x1, y1, xt, yt, _sgp2_cwin->view) == TRUE) {
	    _sgp2_stylus (_sgp2_cwin, x1, y1, 0);
	    _sgp2_stylus (_sgp2_cwin, xt, yt, 1);
	}
    }
}


void 
sgp2_mark_abs (double x, double y)
{
    double xndc, yndc;

    if (_sgp2_cwin == NULL)
	return;

    wc_to_ndc (x, y, &xndc, &yndc);
    markndc (_sgp2_cwin, xndc, yndc);
    _sgp2_cwin->curx = x;
    _sgp2_cwin->cury = y;
}


void 
sgp2_mark_rel (double x, double y)
{
    sgp2_mark_abs (x + _sgp2_cwin->curx, y + _sgp2_cwin->cury);
}


void 
sgp2_point_abs (double x, double y)
{
    double xndc, yndc;

    if (_sgp2_cwin == NULL)
	return;;

    wc_to_ndc (x, y, &xndc, &yndc);
    pntndc (_sgp2_cwin, xndc, yndc);
    _sgp2_cwin->curx = x;
    _sgp2_cwin->cury = y;
}


void 
sgp2_point_rel (double x, double y)
{
    sgp2_point_abs (x + _sgp2_cwin->curx, y + _sgp2_cwin->cury);
}


/*----------------------------------------------------------------------*/
/*		     Current Transformation Matrix Routine		*/
/*----------------------------------------------------------------------*/


/* NAME
 *	ctm_clr_2			Clear current transformation matrix
 *
 * SYNOPSIS
 *	ctm_clr_2()
 */

void 
ctm_clr_2 (void)
{
    GRFMTX_2D m;

    ident_gmtx_2 (m);
    ctm_set_2 (m);
}


/* NAME
 *	ctm_get_2			Get ctm
 *
 * SYNOPSIS
 *	ctm_get_2 (m)
 * OUT	GRFMTX_2D m			Copy of ctm
 */

void 
ctm_get_2 (GRFMTX_2D m)
{
    int x, y;

    if (_sgp2_cwin == NULL)
	return;

    for (x = 0; x < 3; x++)
	for (y = 0; y < 3; y++)
	    m[x][y] = _sgp2_cwin->ctm_2_x[x][y];
}


/* NAME
 *	ctm_set_2			Set ctm to a matrix
 *
 * SYNOPSIS
 *	ctm_get_ctm_2 (m)
 * IN	GRFMTX m			New ctm
 */

void 
ctm_set_2 (GRFMTX_2D m)
{
    int x, y;
    if (_sgp2_cwin == NULL)
	return;

    for (x = 0; x < 3; x++)
	for (y = 0; y < 3; y++)
	    _sgp2_cwin->ctm_2_x[x][y] = m[x][y];

    _sgp2_cwin->recalc_ndc_to_mc = TRUE;
    _sgp2_cwin->recalc_mc_to_ndc = TRUE;
}


void 
ctm_pre_mult_2 (GRFMTX_2D m)
{
    GRFMTX_2D new_ctm;

    mult_gmtx_2 (m, _sgp2_cwin->ctm_2_x, new_ctm);
    ctm_set_2 (new_ctm);
}


void 
ctm_post_mult_2 (GRFMTX_2D m)
{
    GRFMTX_2D new_ctm;

    mult_gmtx_2 (_sgp2_cwin->ctm_2_x, m, new_ctm);
    ctm_set_2 (new_ctm);
}
